@@ -31,8 +31,6 @@ local schema = {
         token_endpoint = {type = "string", minLength = 1, maxLength = 4096},
         resource_registration_endpoint = {type = "string", minLength = 1, maxLength = 4096},
         client_id = {type = "string", minLength = 1, maxLength = 100},
-        audience = {type = "string", minLength = 1, maxLength = 100,
-                    description = "Deprecated, use `client_id` instead."},
         client_secret = {type = "string", minLength = 1, maxLength = 100},
         grant_type = {
             type = "string",
@@ -73,6 +71,7 @@ local schema = {
             maxLength = 4096
         },
     },
+    required = {"client_id"},
     allOf = {
         -- Require discovery or token endpoint.
         {
@@ -81,13 +80,6 @@ local schema = {
                 {required = {"token_endpoint"}}
             }
         },
-        -- Require client_id or audience.
-        {
-            anyOf = {
-                {required = {"client_id"}},
-                {required = {"audience"}}
-            }
-        },
         -- If lazy_load_paths is true, require discovery or resource registration endpoint.
         {
             anyOf = {
@@ -120,28 +112,10 @@ local _M = {
 
 
 function _M.check_schema(conf)
-    -- Check for deprecated audience attribute and emit warnings if used.
-    if conf.audience then
-        log.warn("Plugin attribute `audience` is deprecated, use `client_id` instead.")
-        if conf.client_id then
-            log.warn("Ignoring `audience` attribute in favor of `client_id`.")
-        end
-    end
     return core.schema.check(schema, conf)
 end
 
 
--- Return the configured client ID parameter.
-local function authz_keycloak_get_client_id(conf)
-    if conf.client_id then
-        -- Prefer client_id, if given.
-        return conf.client_id
-    end
-
-    return conf.audience
-end
-
-
 -- Some auxiliary functions below heavily inspired by the excellent
 -- lua-resty-openidc module; see https://github.com/zmartzone/lua-resty-openidc
 
@@ -339,7 +313,7 @@ end
 
 -- Ensure a valid service account access token is available for the configured client.
 local function authz_keycloak_ensure_sa_access_token(conf)
-    local client_id = authz_keycloak_get_client_id(conf)
+    local client_id = conf.client_id
     local ttl = conf.cache_ttl_seconds
     local token_endpoint = authz_keycloak_get_token_endpoint(conf)
 
@@ -648,7 +622,7 @@ local function evaluate_permissions(conf, ctx, token)
         method = "POST",
         body = ngx.encode_args({
             grant_type = conf.grant_type,
-            audience = authz_keycloak_get_client_id(conf),
+            audience = conf.client_id,
             response_mode = "decision",
             permission = permission
         }),
@@ -732,7 +706,7 @@ local function generate_token_using_password_grant(conf,ctx)
         return 422, {message = err}
     end
 
-    local client_id = authz_keycloak_get_client_id(conf)
+    local client_id = conf.client_id
 
     local token_endpoint = authz_keycloak_get_token_endpoint(conf)
 
