@@ -75,9 +75,6 @@ if is_apisix_or then
             return nil, err
         end
 
-        -- it will case output problem with buffer when log is larger than buffer
-        file:setvbuf("no")
-
         handler.file = file
         handler.open_time = ngx.now() * 1000
         return handler
@@ -119,14 +116,11 @@ local function write_file_data(conf, log_message)
     if not file then
         core.log.error("failed to open file: ", conf.path, ", error info: ", err)
     else
-        -- file:write(msg, "\n") will call fwrite several times
-        -- which will cause problem with the log output
-        -- it should be atomic
-        msg = msg .. "\n"
-        -- write to file directly, no need flush
-        local ok, err = file:write(msg)
+        local ok, err = file:write(msg, '\n')
         if not ok then
             core.log.error("failed to write file: ", conf.path, ", error info: ", err)
+        else
+            file:flush()
         end
 
         -- file will be closed by gc, if open_file_cache exists
