@@ -68,11 +68,6 @@ local schema = {
             type        = "string",
             pattern     = [[^[0-9a-zA-Z-.]+(:\d{1,5})?$]],
         },
-        scheme = {
-            description = "new scheme for upstream",
-            type    = "string",
-            enum    = {"http", "https"}
-        },
         headers = {
             description = "new headers for request",
             type = "object",
@@ -161,28 +156,14 @@ function _M.rewrite(conf, ctx)
             ctx.var[upstream_vars[name]] = conf[name]
         end
     end
-    if conf["scheme"] then
-        ctx.upstream_scheme = conf["scheme"]
-    end
-
-
-    local function escape_separator(s)
-        return re_sub(s, [[\?]], "%3F", "jo")
-    end
 
     local upstream_uri = ctx.var.uri
-    local separator_escaped = false
     if conf.use_real_request_uri_unsafe then
         upstream_uri = ctx.var.real_request_uri
     elseif conf.uri ~= nil then
-        separator_escaped = true
-        upstream_uri = core.utils.resolve_var(conf.uri, ctx.var, escape_separator)
+        upstream_uri = core.utils.resolve_var(conf.uri, ctx.var)
     elseif conf.regex_uri ~= nil then
-        if not str_find(upstream_uri, "?") then
-            separator_escaped = true
-        end
-
-        local uri, _, err = re_sub(upstream_uri, conf.regex_uri[1],
+        local uri, _, err = re_sub(ctx.var.uri, conf.regex_uri[1],
                                    conf.regex_uri[2], "jo")
         if uri then
             upstream_uri = uri
@@ -196,17 +177,11 @@ function _M.rewrite(conf, ctx)
     end
 
     if not conf.use_real_request_uri_unsafe then
-        local index
-        if separator_escaped then
-            index = str_find(upstream_uri, "?")
-        end
-
+        local index = str_find(upstream_uri, "?")
         if index then
-            upstream_uri = core.utils.uri_safe_encode(sub_str(upstream_uri, 1, index - 1)) ..
-                sub_str(upstream_uri, index)
+            upstream_uri = core.utils.uri_safe_encode(sub_str(upstream_uri, 1, index-1)) ..
+                           sub_str(upstream_uri, index)
         else
-            -- The '?' may come from client request '%3f' when we use ngx.var.uri directly or
-            -- via regex_uri
             upstream_uri = core.utils.uri_safe_encode(upstream_uri)
         end
 
