@@ -73,6 +73,28 @@ local function get_custom_format_log(ctx, format)
 end
 _M.get_custom_format_log = get_custom_format_log
 
+
+local function latency_details_in_ms(ctx)
+    local latency = (ngx_now() - ngx.req.start_time()) * 1000
+    local upstream_latency, apisix_latency = nil, latency
+
+    if ctx.var.upstream_response_time then
+        upstream_latency = ctx.var.upstream_response_time * 1000
+        apisix_latency = apisix_latency - upstream_latency
+
+        -- The latency might be negative, as Nginx uses different time measurements in
+        -- different metrics.
+        -- See https://github.com/apache/apisix/issues/5146#issuecomment-928919399
+        if apisix_latency < 0 then
+            apisix_latency = 0
+        end
+    end
+
+    return latency, upstream_latency, apisix_latency
+end
+_M.latency_details_in_ms = latency_details_in_ms
+
+
 local function get_full_log(ngx, conf)
     local ctx = ngx.ctx.api_ctx
     local var = ctx.var
@@ -96,6 +118,8 @@ local function get_full_log(ngx, conf)
         }
     end
 
+    local latency, upstream_latency, apisix_latency = latency_details_in_ms(ctx)
+
     local log =  {
         request = {
             url = url,
@@ -120,7 +144,9 @@ local function get_full_log(ngx, conf)
         consumer = consumer,
         client_ip = core.request.get_remote_client_ip(ngx.ctx.api_ctx),
         start_time = ngx.req.start_time() * 1000,
-        latency = (ngx_now() - ngx.req.start_time()) * 1000
+        latency = latency,
+        upstream_latency = upstream_latency,
+        apisix_latency = apisix_latency
     }
 
     if ctx.resp_body then
@@ -185,26 +211,6 @@ function _M.get_req_original(ctx, conf)
 end
 
 
-function _M.latency_details_in_ms(ctx)
-    local latency = (ngx_now() - ngx.req.start_time()) * 1000
-    local upstream_latency, apisix_latency = nil, latency
-
-    if ctx.var.upstream_response_time then
-        upstream_latency = ctx.var.upstream_response_time * 1000
-        apisix_latency = apisix_latency - upstream_latency
-
-        -- The latency might be negative, as Nginx uses different time measurements in
-        -- different metrics.
-        -- See https://github.com/apache/apisix/issues/5146#issuecomment-928919399
-        if apisix_latency < 0 then
-            apisix_latency = 0
-        end
-    end
-
-    return latency, upstream_latency, apisix_latency
-end
-
-
 function _M.check_log_schema(conf)
     if conf.include_req_body_expr then
         local ok, err = expr.new(conf.include_req_body_expr)
